// Copyright 2023 Maciej Sobczak
// This code is an idea demonstrator, not a final "product" and
// you are encouraged to improve it according to your actual needs.
// This software is free, like in the Boost Software License:
// https://opensource.org/licenses/BSL-1.0

#ifndef STREAM_DB_INCLUDED
#define STREAM_DB_INCLUDED

#include <cstdio>
#include <string>
#include <vector>

namespace stream_db
{

enum result { ok, io_error, parse_error, callback_error, bad_state };

enum access_mode { read_only, read_write, write_from_scratch };

class store
{
public:
    store();
    virtual ~store();

    result open(const std::string & file_name, access_mode mode = read_write);
    
    result replay();

    result append_record(const std::string & cmd, const std::vector<std::string> & words);
    result process_record(const std::string & cmd, const std::vector<std::string> & words);
    
    result sync();
    void close();
    
private:
    virtual result replay_add(const std::vector<std::string> & words);
    virtual result replay_remove(const std::vector<std::string> & words);
    virtual result replay_modify(const std::vector<std::string> & words);
    
    virtual result replay_custom_command(const std::string & cmd, const std::vector<std::string> & words);
        
    std::FILE * file_;
    access_mode mode_;
};

} // namespace stream_db

#endif // STREAM_DB_INCLUDED
